@@ -8,7 +8,6 @@ import mimetypes
 import zipfile
 
 import codecs
-from django.contrib.auth import get_user_model
 from django.contrib.syndication.views import Feed
 from django.core.files.uploadedfile import UploadedFile
 import os
@@ -27,12 +26,14 @@ from django.db.models import F, Count, Q
 from django.http import HttpResponseRedirect, Http404, HttpResponse, HttpResponseNotModified, StreamingHttpResponse
 from django.shortcuts import render_to_response, get_object_or_404, redirect
 from django.template import RequestContext
+from django.utils.safestring import mark_safe
 from django.utils.text import slugify
 from django.utils.timezone import utc
 from django.utils.translation import ugettext_lazy as _
 from django.views.decorators.csrf import csrf_exempt
 from django.views.static import was_modified_since
 from elasticsearch.exceptions import ConnectionError as ESError
+import markdown
 
 from updoc.djangoproject.settings import strip_split
 from updoc.indexation import search_archive
@@ -317,15 +318,33 @@ def show_doc(request, doc_id, path=''):
     if not created:
         now = datetime.datetime.utcnow().replace(tzinfo=utc)
         LastDocs.objects.filter(user=user, doc=doc).update(count=F('count') + 1, last=now)
+    editable = request.user.is_superuser or request.user == doc.user
     if not os.path.isfile(full_path):
         directory = list_directory(root_path, path, view, view_arg='path',
                                    view_kwargs={'doc_id': doc.id}, dir_view_name=view,
                                    dir_view_arg='path', dir_view_kwargs={'doc_id': doc.id}, show_files=True,
                                    show_dirs=True, show_parent=True, show_hidden=False)
-        editable = request.user.is_superuser or request.user == doc.user
         template_values = {'directory': directory, 'doc': doc, 'editable': editable, 'title': str(doc),
                            'keywords': ' '.join([keyword.value for keyword in doc.keywords.all()]), 'doc_id': doc_id, }
         return render_to_response('list_dir.html', template_values, RequestContext(request))
+    if full_path.endswith('.md'):
+        view_name = 'updoc.views.show_doc'
+        if request.user.is_anonymous():
+            view_name = 'updoc.views.show_doc_alt'
+        path_components = [(reverse(view_name, kwargs={'doc_id': doc_id, 'path': ''}), _('root'))]
+        components = path.split('/')
+        for index_, comp in enumerate(components):
+            p = '/'.join(components[0:index_ + 1])
+            path_components.append((reverse(view_name, kwargs={'doc_id': doc_id, 'path': p}), comp))
+        template_values = {'doc': doc, 'editable': editable, 'title': str(doc), 'path': path, 'paths': path_components,
+                           'keywords': ' '.join([keyword.value for keyword in doc.keywords.all()]), 'doc_id': doc_id, }
+        try:
+            with open(full_path) as fd:
+                content = fd.read()
+            template_values['content'] = mark_safe(markdown.markdown(content))
+            return render_to_response('markdown.html', template_values, RequestContext(request))
+        except UnicodeDecodeError:
+            pass
     return sendfile(request, full_path, allow_replace=True)
 
 
@@ -368,18 +387,20 @@ def show_proxies(request):
 def upload_api(request):
     user = request.user if request.user.is_authenticated() else None
     if user is None:
-        username = request.GET.get('username', '')
-        password = request.GET.get('password', '')
-        if username and password:
-            users = get_user_model().objects.filter(username=username)[0:1]
-            if not users:
-                return HttpResponse(_('Invalid user %(u)s') % {'u': username}, status=401)
-            user = users[0]
-            if not user.check_password(password):
-                return HttpResponse(_('Invalid password for user %(u)s') % {'u': username})
-        else:
-            return HttpResponse(_('You must be logged to upload files. You can provide your credentials by adding '
-                                  '&username=USERNAME&password=PASSWORD to your request'), status=401)
+        # username = request.GET.get('username', '')
+        # password = request.GET.get('password', '')
+        # username = unquote(username)
+        # password = unquote(password)
+        # if username and password:
+        #     users = get_user_model().objects.filter(username=username)[0:1]
+        #     if not users:
+        #         return HttpResponse(_('Invalid user %(u)s') % {'u': username}, status=401)
+        #     user = users[0]
+        #     if not user.check_password(password):
+        #         return HttpResponse(_('Invalid password for user %(u)s') % {'u': username}, status=401)
+        #     request.user = user
+        # else:
+        return HttpResponse(_('You must be logged to upload files.'), status=401)
     elif request.method != 'POST':
         return HttpResponse(_('Only POST requests are allowed.'), status=400)
     form = UploadApiForm(request.GET)
@@ -400,10 +421,13 @@ def upload_api(request):
     # ok, we have the tmp file
     uploaded_file = UploadedFile(name=form.cleaned_data['filename'], file=tmp_file)
 
-    for existing_obj in UploadDoc.objects.filter(user=user, name=form.cleaned_data['name']):
-        existing_obj.delete()
+    existing_obj = None
+    existing_objs = list(UploadDoc.objects.filter(user=user, name=form.cleaned_data['name'])[0:1])
+    if existing_objs:
+        existing_obj = existing_objs[0]
+        existing_obj.clean_archive()
     try:
-        obj = process_new_file(uploaded_file, request)
+        obj = process_new_file(uploaded_file, request, obj=existing_obj)
         obj.name = form.cleaned_data['name']
         obj.save()
         for keyword in strip_split(form.cleaned_data['keywords'].lower()):
@@ -415,9 +439,27 @@ def upload_api(request):
     return HttpResponse(_('File successfully uploaded and indexed.'), status=200)
 
 
+def full_list(request):
+    user = request.user if request.user.is_authenticated() else None
+    search = DocSearchForm(request.GET)
+    if request.user.is_anonymous() and not settings.PUBLIC_INDEX:
+        messages.info(request, _('You must be logged to see documentations.'))
+        keywords_with_counts, recent_uploads, recent_checked = [], [], []
+    else:
+        recent_uploads = UploadDoc.objects.order_by('name')
+        recent_checked = LastDocs.objects.filter(user=user).select_related().order_by('-last')[0:40]
+    if request.user.is_anonymous() and not settings.PUBLIC_BOOKMARKS:
+        rss_roots = []
+    else:
+        rss_roots = RssRoot.objects.all().order_by('name')
+    template_values = {'recent_checked': recent_checked, 'title': _('Updoc'), 'rss_roots': rss_roots,
+                       'recent_uploads': recent_uploads, 'search': search, 'keywords': [],
+                       'list_title': _('All documents'), }
+    return render_to_response('index.html', template_values, RequestContext(request))
+
+
 def index(request):
     """Index view, displaying and processing a form."""
-
     user = request.user if request.user.is_authenticated() else None
     search = DocSearchForm(request.GET)
     if search.is_valid():
@@ -471,7 +513,8 @@ def index(request):
     else:
         rss_roots = RssRoot.objects.all().order_by('name')
     template_values = {'recent_checked': recent_checked, 'title': _('Updoc'), 'rss_roots': rss_roots,
-                       'recent_uploads': recent_uploads, 'search': search, 'keywords': keywords_with_counts}
+                       'recent_uploads': recent_uploads, 'search': search, 'keywords': keywords_with_counts,
+                       'list_title': _('Recent uploads'), }
     return render_to_response('index.html', template_values, RequestContext(request))
 
 
